@@ -1,18 +1,41 @@
-use crate::cmd::Command;
+use crate::cmd::{
+    Command,
+    COMMAND_CLASSIFIER,
+};
 
 /// Tokens understood by the parser.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Token<'a> {
-    Invalid,
+    Invalid(char),
     Id(&'a str),
     Def,
     Gives,
     Lambda,
     OpenParen,
     CloseParen,
+    Strict,
     Command(Command),
 }
 
+/// Removes whitespace and line continuation token (if any) from the end
+/// of a line, returning whether there was a line continuation token.
+///
+pub fn strip_whitespace_and_line_cont(line: &mut String) -> bool {
+    while let Some(c) = line.chars().next_back() {
+        if !c.is_whitespace() {
+            break;
+        }
+        line.pop();
+    }
+    match line.chars().next_back() {
+        Some('&') => {
+            line.pop();
+            true
+        },
+        _ => false,
+    }
+}
+
 /// An iterator over the tokens of a string. Used by Parser.
 ///
 /// These tokens are all considered the same:
@@ -87,7 +110,7 @@ impl<'a> Iterator for TokenIter<'a> {
                             Some(Token::Command(cmd))
                         }
                     },
-                    None => Some(Token::Invalid),
+                    None => Some(Token::Invalid(first_char)),
                 };
             } else if first_char == '#' {
                 // comment; skip to the end of line.
@@ -105,6 +128,7 @@ impl<'a> Iterator for TokenIter<'a> {
             (vec!["λ", "\\", "lambda"], Token::Lambda),
             (vec!["("],                 Token::OpenParen),
             (vec![")"],                 Token::CloseParen),
+            (vec!["!"],                 Token::Strict),
             // anything else is either a Id or an invalid token.
         ];
         for (names, token_class) in classifier {
@@ -139,7 +163,7 @@ impl<'a> TokenIter<'a> {
     fn consume_whitespace(&mut self) {
         let rest_of_string = self.rest_of_string();
         for c in rest_of_string.chars() {
-            if !c.is_whitespace() {
+            if !c.is_whitespace() && c != '\u{0}' {
                 break
             }
             self.pos += c.len_utf8();
@@ -152,15 +176,15 @@ impl<'a> TokenIter<'a> {
             panic!("TokenIter.next(): rest_of_string is empty");
         }
 
-        let (name_len, last_char_len) = self.get_next_word_and_last_char_len();
+        let (name_len, last_char) = self.get_next_word_and_last_char();
 
         if name_len == 0 {
             // we must increment position here too, otherwise we'll yield
             // the same invalid token forever.
             // Note that each invalid token we yield has 1 char only.
             //
-            self.pos += last_char_len;
-            Token::Invalid
+            self.pos += last_char.len_utf8();
+            Token::Invalid(last_char)
         } else {
             self.pos += name_len;
             Token::Id(&rest_of_string[0..name_len])
@@ -169,14 +193,14 @@ impl<'a> TokenIter<'a> {
 
     fn get_command(&mut self) -> Option<Command> {
         let rest_of_string = self.rest_of_string();
-        if rest_of_string.chars().next().unwrap() == '=' {
+        if let Some('=') = rest_of_string.chars().next() {
             self.pos += 1;
             return Some(Command::Define); // ":="
         }
 
-        let (name_len, _) = self.get_next_word_and_last_char_len();
+        let (name_len, _) = self.get_next_word_and_last_char();
         let name = &rest_of_string[0..name_len];
-        for class in crate::cmd::COMMAND_CLASSIFIER {
+        for class in COMMAND_CLASSIFIER {
             if name == class.short_name || name == class.long_name {
                 self.pos += name.len();
                 self.cmd_arg_expected = class.arg_expected;
@@ -186,19 +210,19 @@ impl<'a> TokenIter<'a> {
         None
     }
 
-    fn get_next_word_and_last_char_len(&self) -> (usize, usize) {
+    fn get_next_word_and_last_char(&self) -> (usize, char) {
         let rest_of_string = self.rest_of_string();
         let mut name_len = 0;
-        let mut last_char_len = 0;
+        let mut last_char:char = '\0';
 
         for c in rest_of_string.chars() {
-            last_char_len = c.len_utf8();
-            if !c.is_alphabetic() && c != '_' {
+            last_char = c;
+            if !c.is_alphanumeric() && c != '_' {
                 break
             }
-            name_len += last_char_len;
+            name_len += c.len_utf8();
         }
-        (name_len, last_char_len)
+        (name_len, last_char)
     }
 }
 
@@ -323,4 +347,15 @@ mod tests {
         assert_eq!(iter.next(), None);
     }
 
+    #[test]
+    fn strict_var() {
+        let s = "\\!x -> x";
+        let mut iter = TokenIter::new(s);
+        assert_eq!(iter.next(), Some(Token::Lambda));
+        assert_eq!(iter.next(), Some(Token::Strict));
+        assert_eq!(iter.next(), Some(Token::Id("x")));
+        assert_eq!(iter.next(), Some(Token::Gives));
+        assert_eq!(iter.next(), Some(Token::Id("x")));
+        assert_eq!(iter.next(), None);
+    }
 }
